#ifndef BmlLalHodoscopeReadout_HH
#define BmlLalHodoscopeReadout_HH

#include <iostream>
#include <fstream>

#include "RcdHeader.hh"
#include "RcdUserRW.hh"
#include "SubAccessor.hh"
#include "SubInserter.hh"


class BmlLalHodoscopeReadout : public RcdUserRW {

public:
  BmlLalHodoscopeReadout(const std::string &skt) : _readout(false) {
  }

  virtual ~BmlLalHodoscopeReadout() {
  }

  virtual bool record(RcdRecord &r) {
    if(doPrint(r.recordType())) {
      std::cout << std::endl << "BmlLalHodoscopeReadout::record()" << std::endl;
      r.RcdHeader::print(std::cout," ");
    }

    if(r.recordType()==RcdHeader::runStart) {

      // Access the DaqRunStart to get print level 
      SubAccessor accessor(r);
      std::vector<const DaqRunStart*> v(accessor.extract<DaqRunStart>());

      //TEMP!!!!

      //      if(v.size()==1)  _printLevel=v[0]->runType().printLevel();

      // Add in some dummy data
      SubInserter inserter(r);
      inserter.insert<BmlLalHodoscopeRunData>(true);
    }

    if(r.recordType()==RcdHeader::runEnd) {
      // Add in some dummy data
      SubInserter inserter(r);
      inserter.insert<BmlLalHodoscopeRunData>(true);
    }

    if(r.recordType()==RcdHeader::configurationStart) {
      SubAccessor accessor(r);

      std::vector<const BmlLalHodoscopeConfigurationData*>
        v(accessor.extract<BmlLalHodoscopeConfigurationData>());
 
      if(doPrint(r.recordType(),1)) {
        std::cout << " Number of BmlLalHodoscopeConfigurationData subrecords accessed = "
                  << v.size() << std::endl;
        for(unsigned i(0);i<v.size();i++) {
          v[i]->print(std::cout," ") << std::endl;
        }
      }

      _readout=(v.size()==1);
  
      // Add in some dummy data
      if(_readout) {
	SubInserter inserter(r);
	BmlLalHodoscopeConfigurationData c;
	c.timeStamp(time(0));
	c.dummy(v[0]->dummy());
	inserter.insert<BmlLalHodoscopeConfigurationData>(c);
      }
    }

    if(r.recordType()==RcdHeader::configurationEnd) {

      // Add in some dummy data
      if(_readout) {
	SubInserter inserter(r);
	BmlLalHodoscopeConfigurationData c;
	c.timeStamp(time(0));
	inserter.insert<BmlLalHodoscopeConfigurationData>(c);
      }
    }
    
    if(r.recordType()==RcdHeader::trigger) {

      // Add in some dummy data
      if(_readout) {
	SubInserter inserter(r);
	inserter.insert<BmlLalHodoscopeTriggerData>(true);
      }
    }

    if(r.recordType()==RcdHeader::event) {

      // Add in some dummy data
      //if(_readout) {
	SubInserter inserter(r);
	BmlLalHodoscopeEventData
	  *d(inserter.insert<BmlLalHodoscopeEventData>(true));
	inserter.extend(4*d->numberOfAnalogueHits());
	//}
    }

    return true;
  }

private:
  bool _readout;
};

#endif
