#ifndef MpsLaserReadout_HH
#define MpsLaserReadout_HH

#include <iostream>
#include <fstream>

// dual/inc/utl
#include "UtlPack.hh"

// dual/inc/rcd
#include "RcdUserRW.hh"
#include "RcdHeader.hh"

// dual/inc/sub
#include "SubInserter.hh"
#include "SubAccessor.hh"

// dual/inc/skt
#include "DuplexSocket.hh"


class MpsLaserReadout : public RcdUserRW {

public:
  MpsLaserReadout(std::string ipAddress="localhost",
  //MpsLaserReadout(std::string ipAddress="131.169.184.160", // tent at DESY
  //MpsLaserReadout(std::string ipAddress="131.169.184.162", // beam at DESY
		  unsigned port=1201, unsigned tries=10) :
    RcdUserRW(), _socket(ipAddress.c_str(),port,tries) {

    std::cout << std::endl << "MpsLaserReadout::ctor()  SOCKET ESTABLISHED" << std::endl << std::endl;
  }

  virtual ~MpsLaserReadout() {
  }

  bool record(RcdRecord &r) {
    if(doPrint(r.recordType())) {
      std::cout << "MpsLaserReadout::record()" << std::endl;
      r.RcdHeader::print(std::cout," ") << std::endl;
    }
    
    // Check record type
    if(r.recordType()==RcdHeader::startUp) {

      std::string recv;
      sendAndRecv("stup",recv);
      
      // Nothing to read back
    }
    
    if(r.recordType()==RcdHeader::runStart ||
       r.recordType()==RcdHeader::runEnd) {
      std::string recv;
      sendAndRecv("runn",recv);

      MpsLaserRunData x;
      assert(x.parse(recv));

      if(doPrint(r.recordType(),1)) {
	x.print(std::cout," ") << std::endl;
      }

      SubInserter inserter(r);
      inserter.insert<MpsLaserRunData>(x);
    }

    if(r.recordType()==RcdHeader::configurationStart) {
      std::string recv;
      
      SubAccessor accessor(r);
      std::vector<const MpsLaserConfigurationData*>
	v(accessor.access<MpsLaserConfigurationData>());
      assert(v.size()<=1);
      
      // Check for data to write down to PC
      if(v.size()==1) {
	if(doPrint(r.recordType(),1)) {
	  v[0]->print(std::cout," ") << std::endl;
	}
	sendAndRecv(std::string("cfgw")+v[0]->parse(),recv);
      } else {
	sendAndRecv("cfgr",recv);
      }

      MpsLaserConfigurationData x;
      assert(x.parse(recv));

      if(doPrint(r.recordType(),1)) {
	x.print(std::cout," ") << std::endl;
      }

      SubInserter inserter(r);
      inserter.insert<MpsLaserConfigurationData>(x);
    }
    
    if(r.recordType()==RcdHeader::configurationEnd) {
      std::string recv;
      sendAndRecv("cfgr",recv);

      MpsLaserConfigurationData x;
      assert(x.parse(recv));

      if(doPrint(r.recordType(),1)) {
	x.print(std::cout," ") << std::endl;
      }

      SubInserter inserter(r);
      inserter.insert<MpsLaserConfigurationData>(x);
    }

    return true;
  }
  
  bool sendAndRecv(const std::string &s, std::string &r) {
    r.clear();

    //if(doPrint(r.recordType(),4))
    //  std::cout << " sendAndRecv()  std::string to send "
    //	<< s.size() << " bytes ==>" << s << "<==" << std::endl;

    std::string t("daqrequest "+s+'#');
    //if(doPrint(r.recordType(),5))
 std::cout << t << std::endl;

    if(!_socket.send(t.c_str(),t.size())) {
      std::cerr << "MpsLaserReadout::sendAndRecv()  Error writing to socket;"
		<< " number of bytes written < " << s.size() << std::endl;
      perror(0);
      return false;
    }
    
    bool terminated(false);
    //bool terminated(true);
    char x;
    std::string h;
    //std::cout <<"Before receving part! " << terminated << std::endl;
     
    for(unsigned i(0);!terminated;i++) {
      int n(-1);
      //if(doPrint(r.recordType(),6)) std::cout << " sendAndRecv()  Waiting for byte " << i;
      n=_socket.recv(&x,1);
      //if(doPrint(r.recordType(),6)) std::cout << "...got byte " << i << std::endl;
      
      if(n!=1) {
	std::cerr << "MpsLaserReadout::sendAndRecv()  Error reading from socket;"
		  << " number of bytes written = " << n << " < 1" << std::endl;
	perror(0);
	return false;
      }

      if(x=='#') terminated=true;
      else {
	if(i>10) r+=x;
	else     h+=x;
      }
    }

    //if(doPrint(r.recordType(),4))

      std::cout << " sendAndRecv()  std::string recv ed "
    	<< " header " << h.size()
    	<< " bytes ==>" << h << "<=="
    	<< ", message " << r.size()
    	<< " bytes ==>" << r << "<==" << std::endl;
    
    return true;
  }
    
private:
  DuplexSocket _socket;
  MpsLaserRunData _laserData;
};

#endif
